//
//  Examples
//  Guit GUI Toolkis
//  Copyright © 2019/2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include <gshape.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;


class Misc : public GWindow {
  GText status{"Ready"};

public:
  Misc() : GWindow("#misc Misc.")
  {
    *this
    << (MenuBar()
        << ("Menu1" << Menu1())
        << ("Menu2" << Menu2())
        << ("Menu3" << Menu3())
        << ("Menu4" << Menu4())
        )
    << (Table()
        << (TRow()
            << Conditions1()
            << Conditions2()
            << BoundProps()
            )
        << (TRow()
            << Navigation()
            << SyncedList()
            << Guards()
            )
        )
        << (StatusBar << (Label << status));
  }

private:

  GEventFunction doIt = [this](GEvent& e) {
    status = e.chosen() ? e.chosen()->text() : "None";
  };

  GMenu& Menu1() {
    return Menu
    << doIt
    << "Copy"
    << "Paste"
    << "Delete";
  }

  GMenu& Menu2() {
    return Menu("{items: Copy, Paste, Delete}") << doIt;  // embeded CSS
  }

  GMenu& Menu3() {
    return Menu("#menu3") << doIt;   // items specified in CSS style
  }
  
  GMenu& Menu4() {
    return Menu
    << ("Copy"   << [this]{status = "Copy";})
    << ("Paste"  << [this]{status = "Paste";})
    << ("Delete" << [this]{status = "Delete";});
  }

  GBox& Menus() {
    return Pane("<b>Menus")
    << (Button("Menu 1") << Menu1())
    << (Button("Menu 2") << Menu2())
    << (Button("Menu 3") << Menu3())
    << (Button("Menu 4") << Menu4());
  }


  // - - - - -

  GBox& Conditions1() {
    return Pane("<b>Conditions 1")
    << (Button(" Press Me ")
        << [this]{status = "Action";}
        << on::mousedown / [this]{status = "MouseDown";}
        << on::mouseup   / [this]{status = "MouseUp";}
        )
    << (Button(" Press Me ")
        << on::mousedown / "<c:yellow>Mouse Down"
        << on::mouseup   / "<c:blue>Mouse Up"
        )
    << (Button
        << on::checked   / "<b>I am selected"
        << on::unchecked / "<i>I am not selected"
        );
  }

  // - - - - -

  GBox& Conditions2() {
    auto& speed = *new GFloat();
    speed.setRange(0,200);

    return Pane("<b>Active Conditions")
    << (HBox
        << "0"s << (Slider << speed) << "200"
        )
    << ""
    << (Label("{font:bold; align:c c; format:2}")
        << (~speed == 0)  / "Stopped"
        << (~speed > 0 && ~speed <= 90)  / ("<c=green>Safe: " + ~speed +" km/h")
        << (~speed > 9 && ~speed <= 130) / ("<c=blue>Fast: "+ ~speed +" km/h")
        << (~speed > 130) / ("<c=red>Danger! "+ ~speed +" km/h")
        );
  }

  // - - - - -

  GBox& SyncedList() {
    enum {None, Ok, Warning, Danger};
    auto& choice = Choice(None);
    auto& choice_text = Text();

    return Pane("<b>Synced List")
    << (~choice == None) / GBackground::none
    << (~choice == Ok)      / Background("green")
    << (~choice == Warning) / Background("orange")
    << (~choice == Danger)  / Background("red")
    << (Table
        << (TRow << "Choice"s << (ListBox("{items: None, Ok, Warning, Danger}")
                            << choice << choice_text
                            )
            )
        << (TRow << "Status"s << choice_text)
        << (TRow << "Value"s  << (TextField << choice))
        );
  }

  // - - - - -

  GBox& BoundProps() {
    auto& stat = Choice(-1);
    auto& card = Int();
    auto& id = Text();

    return Pane("<b>Bound Props")
    << (HBox << "Status:"s
        << (OptionBox("{flow:row; items: Child,Adult,Member}") << stat)
        )
    << (Label << (Show() <<= (~stat == 0)) << "Free entrance")
    << (HBox  << (Show() <<= (~stat == 1)) << "Card #: " << card)
    << (HBox  << (Show() <<= (~stat == 2)) << "Member ID" << id);
  }

  // - - - - -

  GBox& Guards() {
    auto& name = Text();
    auto& stat = Choice(-1);
    auto& id = Text();
    auto& show = Int(-1);

    return Pane("{flex: 0 0} <b>Guards")
    << (Table
        << (TRow
            << "Name:"s
            << name
            )
        << (TRow
            << "Status:"s
            << (RadioBox("{flow:row; items: Visitor,Staff; font:small}")
                << stat << [&]{show = -1;}
                )
            )
        )
    << (HBox
        << (Button("Enter")
            << Guard(~name == "") / [&]{name.blink("Required");}
            << Guard(~stat < 0)   / [&]{stat.blink("Required");}
            << Otherwise(~stat == 0) / [&]{show = 0;}
            << Otherwise(~stat == 1) / [&]{show = 1;}
            )
        << (Button("Reset") << [&]{name = ""; stat = -1;})
        )
    << (Label
        << (Show() <<= (~show == 0))
        << ("Welcome Visitor " + ~name)
        )
    << (HBox
        << (Show() <<= (~show == 1))
        << "Staff ID: "
        << id
        << Button("Go")
        )
    ;
  }

  // - - - - -

  GBox& Navigation() {
    auto& page = Choice(0).setRange(0,2);

    return Pane("{align:x x} <b>Navigation")
    << (StackBox
        << page
        << (Pane("Page 1") << OvalShape("{size: 50 50; color:blue}"))
        << (Pane("Page 2") << RectShape("{size: 50 50; color:red}"))
        << (Pane("Page 3") << OvalShape("{size: 50 50; color:blue}")
            << RectShape("{size: 50 50; color:red;}"))
        )
    << (ButtonBox("{flow:row; align:c c; flex:1 0}")
        << (" < " << [&]{page--;})
        << (" > " << [&]{page++;})
        );
  }
};



GWindow& Examples::makeMisc() {
  return *new Misc();
}
