//
//  Tic Tac Toe
//  Guit GUI Toolkit
//

#include <guit.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;

class TicTacToe : public GWindow {
  GTable grid{"#grid"};     // table containing the cells
  GText result;
  GInt gridsize{3};
  GInt turn{1};
  GBool completed{false};   // true if game ended
  GColor wincolor{"red"};   // color when user wins

public:
  TicTacToe() : GWindow("#tictactoe {flex: 1 1} Tic Tac Toe") {
    makeGrid();
    
    *this
    << grid
    << (Label(".result") << result)
    << (VBox(".controls")
        << (Button(".playagain") << [&]{reset();})
        /* << ""  // bugged!
        << (HBox
            << "Grid Size"s
            << (SpinItem(".gridsize") << gridsize)
            ) */
        );

    gridsize << [this]{makeGrid();};
  }

private:

  Gadget& cell(int row, int col) {
    return *grid.childAt(row, col);
  }


  GString cellValue(int row, int col) {
    return grid.childAt(row, col)->text();
  }


  void makeGrid() {
    reset();
    grid.removeChildren();

    for (int row = 0; row < gridsize; ++row) {
      auto& trow = TRow();
      grid << trow;

      for (int col = 0; col < gridsize; ++col) {
        trow
        << (Button(".cell") << [=]{ cellClicked(row,col); });
      }
    }

    updateLayout(Gadget::Update::Now);
  }


  void reset() {
    completed = false;
    result = "";
    // clear all buttons
    grid.perform( [](Gadget& cell){ cell << "" << GColor::black;} );
  }


  void cellClicked(int row, int col) {
    // game already ended
    if (completed()) return;

    // cell already clicked
    if (cellValue(row,col) != "") return;

    // add X or O to the cell and change turn
    if (turn == 1) {
      cell(row,col) << "X";
      turn = 2;
    }
    else {
      cell(row,col) << "O";
      turn = 1;
    }

    // test if cells are aligned
    if (checkRow(row) || checkColumn(col) || checkDiagonal1() || checkDiagonal2()) {
      completed = true;
      result = "Player " + turn.stringValue() + " wins";
      result.blink();
    }
  }


  bool checkRow(int row) {
    GString value = cellValue(row,0);
    if (value.empty()) return false;

    for (int k = 1; k < gridsize; ++k) {
      if (cellValue(row,k) != value) return false;
    }

    for (int k = 0; k < gridsize; ++k) cell(row,k) << wincolor;
    return true;
  }


  bool checkColumn(int col) {
    GString value = cellValue(0,col);
    if (value.empty()) return false;

    for (int k = 1; k < gridsize; ++k) {
      if (cellValue(k,col) != value) return false;
    }

    for (int k = 0; k < gridsize; ++k) cell(k,col) << wincolor;
    return true;
  }

  bool checkDiagonal1() {
    GString value = cellValue(0,0);
    if (value.empty()) return false;

    for (int k = 1; k < gridsize; ++k) {
      if (cellValue(k,k) != value) return false;
    }

    for (int k = 0; k < gridsize; ++k) cell(k,k) << wincolor;
    return true;
  }


  bool checkDiagonal2() {
    GString value = cellValue(0, gridsize-1);
    if (value.empty()) return false;

    for (int k = 1; k < gridsize; ++k) {
      if (cellValue(k, gridsize-1-k) != value) return false;
    }

    for (int k = 0; k < gridsize; ++k) cell(k, gridsize-1-k) << wincolor;
    return true;
  }
};


GWindow& Examples::makeTicTacToe() {
  return *new TicTacToe;
}
