//
// Gadget Rendering
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_Render_hpp
#define Guit_Render_hpp
#include <ggraphics.hpp>
#include <gflow.hpp>
#include <gflex.hpp>
#include <galign.hpp>
#include <ggap.hpp>
#include <gpos.hpp>
#include <gsize.hpp>
#include <gfont.hpp>
namespace guit {


/// @internal Painting/picking context
/// This class should not be used in client code, use Graphics instead.
class GRenderGraphics : public GGraphics {
public:
  GRenderGraphics(GWindow&);
  bool setClipRect(Gadget&);
  GRect area_, cliprect_;
};


/// @internal Gadget Renderer.
class GRender {
public:
  enum {
    DrawNone = 0, DrawBackground = 1<<0, DrawBorder = 1<<1, DrawText = 1<<2,
    AddMargin = 1<<2,
    DrawAll = (DrawBackground | DrawBorder | DrawText | AddMargin)
  };

  struct StyleNode {
    StyleNode() : level_(0), style_(nullptr) {}
    StyleNode(int level, GStyle* style) : level_(level), style_(style) {}
    int level_;
    GStyle* style_;
  };

  ~GRender();
  GRender() = default;
  GRender(Gadget&, GRender* parent_render);
  GRender(const Gadget&) = delete;

  GRender& operator=(GRender const&);

  Gadget& gadget() {return *obj_;}
  void print(std::ostream& out);

  void addCombinator(GStyle*);
  void applyStyles(Gadget*, GStyle& class_style /*,int pseudo*/);
  void applyInStyles(StyleNode& asc_style, Gadget*, GStyle& desc_style /*,int pseudo*/);

  bool paintGadget(GRenderGraphics&, int options);
  void paintPieBackground(GRenderGraphics&);
  void paintBox(GRenderGraphics&, GBox&);
  void drawText(GRenderGraphics&, float offest);
  void drawHotkey(GRenderGraphics&);
  void drawIcon(GRenderGraphics&);
  void drawCaret(GRenderGraphics&, GString const& text,
                 class GFontValue const&, GTextAttributes const*,
                 size_t begline, ssize_t caretpos, bool richtext,
                 float x, float y, float h);
  void setAutoColor(GRenderGraphics&);
  void setTextAutoColor(GRenderGraphics&);
  void setTextAutoColor(GRenderGraphics&, float pos, float offset);
  static bool formatText(class GFormatValue const&, GString const& from, GString& to);

  Gadget* obj_{};
  GRender* parr_{};  // parent_render
  int level_{};
  size_t own_inscount_{}, child_inscount_{};
  std::vector<StyleNode>* instyles_{};
  GString const* text_{};
  GString const* tip_{};
  GString const* hotkey_{};
  GPoint textpos_;
  GDim   textsize_;
  GFontValue font_;
  GFlowValue flow_;
  GFlexValue flex_;
  GPosValue pos_;
  GSizeValue size_;
  GGapValue gap_;
  GAlignValue align_;
  class GFormatValue*  format_{};
  class GRgba const*   color_{};
  class GRgba const*   background_{};
  class GIconValue*    icon_{};
  class GMarginValue*  margin_{};
  class GBorderValue*  border_{};
  GProp::Specif specif[GPropType::RenderCount]{};
};

}
#endif
