//
// Application Theme
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gtheme.hpp>
#include <gstyle.hpp>
#include <gbox.hpp>
#include <gchoice.hpp>
#include <gborder.hpp>
#include <gmodes.hpp>
#//include <gstates.hpp>
#include <gformat.hpp>
#include <gmodes.hpp>
#include <gtext.hpp>
#include <gcontrol.hpp>
#include <grender.hpp>
using namespace std;
namespace guit {

// !CAUTION: fonts (and thus styles) **MUST** be created AFTER the Native App!
void GDefaultTheme::init() {

  GStyle& gadgetstyle = *new GStyle(0);
  {
    auto& s = gadgetstyle;
    GStyleBase::instance().addCoreStyle("/Gadget", s);
    s.modes_.richText = 1;
    s.modes_.enabled = 1;
    s.modes_.shown = 1;
    s
    << GFlow::row
    << GBorder::none     // needed!
    << new GSize("a a")  // inherited by child styles!
    << new GFlex("1 1")
    << new GAlign("s c")
    << new GGap(3,3);
  }
  {
    auto& s = *new GStyle(gadgetstyle);
    GStyleBase::instance().addCoreStyle("/TextItem",s);
    s.modes_.activable = 1;
    s.modes_.focusable = 1;
    s.modes_.checkable = 1; 
    s.modes_.richText = 0;
    s
    << new GSize("6c a")
    << new GFlex("1 0")
    << GBackground::white
    << new GBorder(GBorder::textedit, 3,3,
                   new GColor(0.77f, 0.77f, 0.77f),   // kind of lightgray
                   &GColor::guitblue);
  }
  {
    auto& s = *new GStyle(gadgetstyle);
    GStyleBase::instance().addCoreStyle("/Window", s);
    s.modes_.shown = 0;
    s.modes_.choiceBehavior = uint8_t(GBox::ChoiceBehavior::Select);
    s
    << GFlow::column
    << new GFlex(0,0,GFlex::SelfWH)
    << new GAlign("x x")
    << new GGap(5,5)
    << GFont::regular   // inherited
    << GColor::black    // inherited
    << new GBackground(0.90f, 0.90f, 0.90f);
  }

  GStyleBase::instance().loadBuffer
  (
   "Gadget {:/Gadget}"

   "/Disabled {color:gray}"

   "Filler {:/Gadget; size: a a; flex: 100 100 parent-flow; margin:1}"

   "Separator {:/Gadget; size: 1 1; flex: 0 1 parent-flow;"
   "border: squared 1 1 lightgray lightgray}"
   "Separator:disabled {:/Disabled}"

   "Label {:/Gadget; margin:3 1; flex: 0 1 parent-flow}"
   "Label:disabled {:/Disabled}"

   "Title {:/Gadget; font:bold; margin:3 7; align:c c; flex: 1 0}"
   "Title:disabled {color:gray}"

   "/Item {:/Gadget; flex: 0 1 parent-flow; modes:+activable}"
   "/ItemChecked {color:white; background:0.24 0.6 1}"
   "/ItemActive {color:white; background:guitblue}"
   "/ItemDisabled {:/Disabled}"
   "/ItemDisabledChecked {color:white; background:lightgray}"

   "Item {:/Item}"
   "Item:checked {:/ItemChecked}"
   "Item:active {:/ItemActive}"
   "Item:disabled {:/ItemDisabled}"
   "Item:disabled:checked {:/ItemDisabledChecked}"

   "Button {:/Item; background:white; align:c c; gap:5; border:rounded 9 3}"
   "Button:checked {:/ItemChecked}"
   "Button:active {:/ItemActive}"
   "Button:disabled {:/ItemDisabled}"
   "Button:disabled:checked {:/ItemDisabledChecked}"

   "LinkItem {:/Item; margin:3 2; align:s c; gap:3; color:guitblue}"
   "LinkItem:checked {:/ItemChecked}"
   "LinkItem:active {:/ItemActive}"
   "LinkItem:disabled {:/ItemDisabled}"
   "LinkItem:disabled:checked {:/ItemDisabledChecked}"

   "/StateItem {:/Gadget; flex: 0 1 parent-flow; margin:3 2; align:s c s; gap:3;"
   "modes:+activable,+checkable}"
   "/StateItemActive {background:lightgray}"

   "StateItem {:/StateItem}"
   "StateItem:active {:/StateItemActive}"
   "StateItem:disabled {:/Disabled}"

   "CheckItem {:/StateItem; icon:check}"
   "CheckItem:active {:/StateItemActive}"
   "CheckItem:disabled {:/Disabled}"
   "CheckItem:disabled:checked {:/Disabled}"

   "RadioItem {:/StateItem; icon:radio}"
   "RadioItem:active {:/StateItemActive}"
   "RadioItem:disabled {:/Disabled}"
   "RadioItem:disabled:checked {:/Disabled}"

   "ChoiceItem {:/StateItem; align:c c; gap:5; border:rounded 2 2 '0.7 0.7 0.7' guitblue}"
   "ChoiceItem:active {:/StateItemActive}"
   "ChoiceItem:checked {background:none}"
   "ChoiceItem:disabled {:/Disabled}"
   "ChoiceItem:disabled:checked {:/Disabled}"

   "ValueItem {:/Gadget; size:13 13; flex:1 0 self-flow; align: c c;"
   "            background:0.7 0.7 0.7; modes:downactivable}"
   "Slider {:ValueItem; border:rounded 0 0}"
   "ScrollBar {:ValueItem; size:9 9; border:rounded 1 1}"

   "ChartItem {:/Gadget; size: 20 20; align:c c;"
   "            background:0.7 0.7 0.7; modes:downactivable}"

   "TextItem {:/TextItem;}"
   "TextItem:disabled {:/Disabled}"
   "TextItem:disabled:checked {:/Disabled}"

   "TextField {:/TextItem;}"
   //"TextField {:/TextItem; flex:0 0}"   pbm ici !
   "TextField:disabled {:/Disabled}"
   "TextField:disabled:checked {:/Disabled}"

   "TextArea {:/TextItem; size:40c a; flex:1 1; modes:multiline}"
   "TextArea:disabled {:/Disabled}"
   "TextArea:disabled:checked {:/Disabled}"

   "SpinItem {:/TextItem; align:s c e}"
   "SpinItem:disabled {:/Disabled}"

   // boxes must be at the end because they need gadget, label, etc.
   "/Box {:/Gadget; flow:column; align:x x; gap:5; modes:+ChoiceSelect}"

   "ChartBox {:/Box; flow:row; align:equal equal; gap:1}"

   "Box {:/Box; margin:3}"
   "VBox {:/Box}"
   "HBox {:/Box; flow:row}"
   "Pane {:/Box; border:rounded}"
   "ScrollBox {:/Box}"

   "ButtonBox {:/Box}"
   "ButtonBox > Item {:Button}"
   "ButtonBox > Item:checked {:/ItemChecked}"
   "ButtonBox > Item:active {:/ItemActive}"
   "ButtonBox > Item:disabled {:/ItemDisabled}"
   "ButtonBox > Item:disabled:checked {:/ItemDisabledChecked}"

   "ChoiceBox {:/Box; modes:+ChoiceSelect; margin:3; gap:2}"
   "ChoiceBox > Item {:ChoiceItem}"
   // !! should be color:inherit
   "ChoiceBox > Item:checked {color:black; background:none}"
   "ChoiceBox > Item:disabled {:/Disabled}"

   "RadioBox {:ChoiceBox; border:rounded}"
   "RadioBox > Item {:RadioItem;}"
   "RadioBox > Item:checked {color:black; background:none}"
   "RadioBox > Item:active {color:black; background:lightgrey}"
   "RadioBox > Item:disabled {:/Disabled}"

   "ListBox {:ChoiceBox; border:none 1 2; align:s x; gap:0; background:white}"
   "ListBox > Item {:/Item; margin:5 2; align:s c; gap:6; background:white;"
   "               modes:+checkable}"
   "ListBox > Item:checked {:/ItemChecked}"
   "ListBox > Item:active {:/ItemActive}"
   "ListBox > Item:disabled {:/ItemDisabled}"
   "ListBox > Item:disabled:checked {:/ItemDisabledChecked}"

   //flow:stack;
   "StackBox {:/Box; align:x x; modes:+ChoiceShow}"

   //ici pbm!!!
   "TabBox {:StackBox; border:rounded; modes:+ChoiceShow}"

   "OptionBox {:/Box; flow:row; flex:1 0; margin:3; background:white}"
   // Item is the selector
   //"OptionBox > Item {size:10c a; flex:1 0; align:s x e; gap:6}"
   "OptionBox > Item {size:10c a; flex:1 0; align:s c e; gap:6}"
   "OptionBox > Item:active {background:white; color:black}" // color:inherit!
   "OptionBox > Item:disabled {:/Disabled}"

   "ComboBox {:OptionBox}"
   //"ComboBox > Item {Margin:1 1; Size:18 a; align:c c c 0; Border:none; Background:white}"
   //"ComboBox > TextField {size:10c a; flex:1 0; align:s x e; gap:6}"
   "ComboBox > TextField {size:10c a; flex:1 0; align:s c e; gap:6}"
   //"ComboBox > Item:disabled {:/Disabled}"

   "Menu {:/Gadget; modes:!shown; background:0.95 0.95 0.95; flow:column; align:s x;"
   "      gap:0; flex:0 0; border:rounded 0 5; modes:menulayout,ChoiceSelect}"
   "Menu Item {:/Item; margin:5 4; align:s c s; gap:6;}"
   "Menu Item:checked {:/ItemChecked}"
   "Menu Item:active {:/ItemActive}"
   "Menu Item:disabled {:/ItemDisabled}"
   "Menu Item:disabled:checked {:/ItemDisabledChecked}"

   "Menu StateItem {:/StateItem; margin:6 3; align:s c s; gap:6}"
   "Menu CheckItem {:/StateItem; icon:check}"
   "Menu RadioItem {:/StateItem; icon:radio}"

   "PieMenu {:Menu; flow:pie; border:none; background:none}"

   "MenuBar {:/Box; flow:row; flex:1 0; border:squared 9 1; background:0.96 0.96 0.96}"
   "MenuBar Item {:/Item; margin:4 5; align:s c; gap:6}"
   "MenuBar Item:checked {:/ItemChecked}"
   "MenuBar Item:active {:/ItemActive}"
   "MenuBar Item:disabled {:/ItemDisabled}"
   "MenuBar Item:disabled:checked {:/ItemDisabledChecked}"

   "ToolBar {:MenuBar; align:x x; gap:3; background:0.85 0.85 0.85; margin:0 2}"
   "ToolBar Item {:/Item; margin:4 5; align: s c; gap:1}"
   //"ToolBar Item:checked {border:squared}"
   "ToolBar Item:active {background:gray}"

   "StatusBar {:ToolBar;}"
   "StatusBar Item {:/Item}"
   "StatusBar Item:checked {border:squared}"
   "StatusBar Item:active {background:gray}"

   //"Table {:/Box; align: x x 2 2}"
   "Table {:/Box; align: s x; gap:2 2}"
   "TRow {:/Box; flow:row; align:x x; gap:2}"
   "THead {:/Box; flow:row; align:x x; gap:2; flex:1 0; font: bold}"

  /* bug
   "/Window {:/Gadget; flow:column; flex:0 0; align:x x 5; "
   "         modes:-shown,+ChoiceSelect;"
   "         font:regular; color:black; background:0.9 0.9 0.9"   // inherited
   "        }"
   */
   "Window {:/Window; margin:5 5}"

   "Dialog {:/Window; margin:10 6}"
   "DialogButtons {:/Box; flow:row; size:a a; flex:1 0; margin:10 10 8 6;"
   "              align:c c; gap:10}"

   "InfoDialog {:/Window; margin:18 12; align:x x; gap:6; size:350 150}"
   "InfoDialog /Message {Font: bold 16px sans-serif}"

   "QuitDialog {:InfoDialog}"
   "QuitDialog /Message {Font: bold 16px sans-serif}"

   "FileDialog {:/Window; margin:0 0 10 6; size: 550 330;}"
   "OpenDialog {:FileDialog}"
   "SaveDialog {:FileDialog}"

   "FileTable {:/Box; margin:3; flex:1 0; align:x x; gap:1 0; background:white}"
   "FileRow {:/Box; flow:row; margin:5 1; flex:1 0; align:x x; gap:1 0}"
   "FileRow:checked {color:white; background:0.24 0.6 1}" // selected
   "FileItem {:/Gadget; margin:2 14 0 0; flex:0 1; modes:+downactivable,checkable}"
   "FileItem:checked {color:white}"
   "FileItem:active {color:white}"

   "/Shape {:/Gadget; flex:0 0; modes:+activable,+downActivable,+checkable}"
   "Shape {:/Shape}"
   "RectShape {:/Shape}"
   "OvalShape {:/Shape}"
   "ArcShape {:/Shape}"
   "LineShape {:/Shape}"
   "FreehandShape {:/Shape}"
   );

}

}
