//
// Menus
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gtrigger.hpp>
#include <gmenu.hpp>
#include <gtext.hpp>
#include <gvalueitem.hpp>
#include <gcontrol.hpp>
#include <gstyle.hpp>
#include <gadgetstyle.hpp>
#include <gnative.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_GADGET(Menu)
GIMPLEMENT_GADGET(PieMenu)

GMenu::GMenu(GadgetType& t, GString const& spec) : GBox(t) {
  // dont call GBox(t,spec) otherwise title displayed inside menu
  if (!spec.empty()) {
    GString title;
    GadgetStyle::initStyle(*this, spec, title);
    if (!title.empty()) *this << new GTitle(title);
  }
}

GMenu::GMenu(GString const& spec)
: GMenu(Type(), spec) {}

GMenu& GMenu::clone(GClone const& cm) {
  auto* g = new GMenu;
  g->initClone(*this, cm);
  return *g;
}

Gadget& GMenu::createDefaultItem(GString const& arg) {
  return *new GItem(arg);
}

void GMenu::show(GPoint const& pos, MenuPlacement pl) {
  pos_ = pos;
  place_ = pl;
  show(true);
}

void GMenu::show(GEvent& e, bool autoplace) {
  parent_ = &e.from();
  if (auto* me = e.toMouseEvent()) pos_ = me->wpos();
  place_ = autoplace ? MenuPlacement::BelowParent : MenuPlacement::BelowMouse;
  show(true);
}

GMenu& GMenu::show(bool state) {
  if (state == modes_.shown) return *this;
  modes_.shown = state;

  if (!omodes_.initialized) topInit();
  updateLayout(Update::Needed);

  if (state) {
    switch (place_) {
      case MenuPlacement::BelowParent:
        if (parent_) moveImpl(parent_->wx(),
                              parent_->wy() + parent_->h(),
                              true);
        break;

      case MenuPlacement::AboveParent:
        if (parent_) moveImpl(parent_->wx(),
                              parent_->wy() + parent_->h(),
                              true);
        break;

      case MenuPlacement::BelowMouse:
        moveImpl(pos_.x, pos_.y, true);
        break;

      case MenuPlacement::AroundMouse:
        moveImpl(pos_.x - w()/2.f, pos_.y - h()/2.f, true);
        break;

        /*
         case Over:
         moveImpl(parent_->windowX() + dx_,
         parent_->windowY() + dy_,
         true);
         resize(parent_->w() - dx_, parent_->h() - dy_);
         break;
         
         case Center:
         moveImpl(parent_->windowX() + parent_->w()/2.f + dx_ - w()/2.f,
         parent_->windowY() + parent_->h()/2.f + dy_ - h()/2.f,
         true);
         break;
         */
    }
  }
  
  // autoclose if menu
  GCore::showOverlay(*this, parent_, state, false, false);

  if ((eventmask_ & GEventType::Show) != 0) {
    GEvent e(state ? GTrigger::show : GTrigger::hide, GCore::elapsedTime());
    doFire(e);
  }
  return *this;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - -
/*
 wmodes_.autoclose = true;
 wmodes_.softwin = true;

 // natwin_ = new GNatWindow(this, false, false);
 // GCore::addWindow(*this);
 // *this << Pos();

 if (!spec.empty()) {
 GString value = GadgetStyle::setStyle(*this, spec);
 if (!value.empty()) *this << Title(value);   //???
 }
 */
// - - - - - - - - - - - - - - - - - - - - - - - - - -

GPieMenu::GPieMenu(GadgetType& t, GString const& spec) : GMenu(t, spec) {
  place_ = MenuPlacement::AroundMouse;
}

GPieMenu::GPieMenu(GString const& spec) : GMenu(Type(), spec) {
  place_ = MenuPlacement::AroundMouse;
}

GPieMenu& GPieMenu::clone(GClone const& cm) {
  auto* g = new GPieMenu;
  g->initClone(*this, cm);
  return *g;
}

void GPieMenu::show(GEvent& e, bool autoplace) {
  parent_ = &e.from();
  if (auto* me = e.toMouseEvent()) pos_ = me->wpos();
  place_ = MenuPlacement::AroundMouse;
  show(true);
}

/*
GMenu::GMenu(GKString& spec, GadgetType* m)
: GWindow(m ? *m : WindowType())
{
  wmodes_.autoclose = true;
  wmodes_.softwin = true;

  // natwin_ = new GNatWindow(this, false, false);
  // GCore::addWindow(*this);
  // *this << Pos();

  if (!spec.empty()) {
    GString value = GadgetStyle::setStyle(*this, spec);
    if (!value.empty()) *this << Title(value);   //???
  }
}
*/

}
