//
// Menus
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_Menu_hpp
#define Guit_Menu_hpp
#include <gwindow.hpp>
namespace guit {

/// Toplevel gadget displayed on top of other gadgets
class GMenu : public GBox {
protected:
  GMenu(GadgetType&, GString const& css);

public:
  /// Placement.
  enum struct MenuPlacement {
    BelowParent, AboveParent, BelowMouse, AroundMouse
  };

  GMenu(GString const& css = "");
  GMenu& clone(GClone const&) override;

  GMenu* toMenu() override {return this;}

  /// shows/hides the gadget.
  GMenu& show(bool state = true) override;

  /// shows the gadget at this position.
  virtual void show(GPoint const& pos, MenuPlacement = MenuPlacement::BelowMouse);

  virtual void show(GEvent&, bool autoplace);

  GPoint pos() const override {return GPoint(frame_.x, frame_.y);}
  float x() const override {return frame_.x;}
  float y() const override {return frame_.y;}

  bool isTopLevel() const override {return true;}
  void setPseudoParent(Gadget* parent) {parent_ = parent;}
  void setPlacement(MenuPlacement placement) {place_ = placement;}

  using GType = GadgetType_<GMenu,GBox>;
  static GType& Type();
  GType& type() const override {return Type();}

  /// @internal
  /// @{
  Gadget& createDefaultItem(GString const& arg) override;
  MenuPlacement place_{};
  GPoint pos_;
  /// @}
};


/// Creates a GMenu.
GMenu& Menu(GString const& css = "");



/// Pie menu.
class GPieMenu : public GMenu {
protected:
  GPieMenu(GadgetType&, GString const& css);

public:
  GPieMenu(GString const& css = "");
  GPieMenu& clone(GClone const&) override;

  using GMenu::show;
  void show(GEvent&, bool autoplace) override;

  using GType = GadgetType_<GPieMenu,GMenu>;
  static GType& Type();
  GType& type() const override {return Type();}
};

/// Creates a GMenu.
GPieMenu& PieMenu(GString const& css = "");

}
#endif
