//
//  Cursors
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gapp.hpp>
#include <gtrigger.hpp>
#include <gcursor.hpp>
#include <ggraphics.hpp>
#include <gadget.hpp>
#include <gwindow.hpp>
#include <gcolor.hpp>
#include <gcontrol.hpp>
#include <grender.hpp>
#include <gnative.hpp>
using namespace std;

namespace guit {

/*
GPropType_<GCursor>& GCursor::Type() {
  static auto& t =  GCore::addPropType<GCursor>("Cursor");
  return t;
}*/
//static constexpr GPropRole Role() {return GPropRole::Special;}
//virtual RoleKind roleKind() const override {return RoleKind::Misc;}

// cant be in heder bec of smart ptrs
GChannel::GChannel(unsigned int id) : GCursor(id) {}


GCursor::GCursor(unsigned int id) : id_(id), color_(&GColor::black) {
  if (id_ == 0) {
    nativecursor_ = true;
    shown_ = true;     // not necessarily true !
  }
  else maxderive_ = 30;
}

GCursor& GCursor::operator<<(GFunction const& fun) {
  props_.addEnd(on::changeaction / fun);
  return *this;
}

GCursor* GCursor::cursor(unsigned int id) {
  return GCore::channel(id);
}

GCursor& GCursor::obtainCursor(unsigned int id) {   // AREVOIR!!!
  return GCore::obtainChannel(id);
}

int GCursor::cursorCount() {
  return int(GCore::channels().size());
}

GDim GCursor::screenSize() {
  return GCore::natApp()->screenSize();
}

GPoint const& GCursor::pos() const {
  if (id_ == 0) pos_ = GCore::natApp()->cursorPos();
  return pos_;
}

GPos& operator<<=(GPos& pos, GCursor& c) {
  c << [&pos,&c] {pos = c.wpos();};
  return pos;
}

GPoint GCursor::wpos() const {
  if (!window_) return GPoint();
  else{
    auto p = pos();
    p.x -= window_->x();
    p.y -= window_->y();
    return p;
  }
}

void GCursor::move(GPoint const& pos) {
  //cerr << "GCursor::move " << pos << " chan " << id_ << endl;
  if (int(pos.x) != int(pos_.x) || int(pos.y) != int(pos_.y)) {
    if (nativecursor_ && id_ != 0) GCore::natApp()->moveCursor(pos);
    pos_ = pos;
    //if (!props_.empty()) props_.fireChangeAction(this);
    if (!props_.empty()) props_.fire(this);
  }
}

void GCursor::channelMove(GMouseEvent& e) {
  //cerr << "GCursor::channelMove " << GPoint(e.wpos_.x + e.window_->x(),
  //e.wpos_.y + e.window_->y()) << " chan " << id_ << endl;
  int x = int(e.wpos_.x + e.window_->x());
  int y = int(e.wpos_.y + e.window_->y());
  if (window_.get() != e.window_ || x != int(pos_.x) || y != int(pos_.y)) {
    window_ = e.window_;
    pos_.x = e.wpos_.x + e.window_->x();
    pos_.y = e.wpos_.y + e.window_->y();
    if (nativecursor_ && id_ != 0) GCore::natApp()->moveCursor(pos_);
    if (!props_.empty()) props_.fire(this);
  }
}

void GCursor::channelDown(GMouseEvent&) {
  pressed_ = true;
}

void GCursor::channelUp(GMouseEvent&) {
  pressed_ = false;
}

void GCursor::setShape(Shape shape) {
  if (nativecursor_) {
    shape_ = shape;
    GCore::natApp()->setCursorShape(shape);
  }
  else gerror("GCursor::setShape","Implemented only for native cursor");
}

void GCursor::setColor(GColor&col) {
  color_ = &col;
  //if (native_) GCore::natApp()->xxx(col);
}

void GCursor::show(bool state) {
  // pbm du hide count a la noix de cocoa!
  if (shown_ == state) return;
  shown_ = state;
  if (nativecursor_) GCore::natApp()->showCursor(state);
}

void GCursor::freeze(bool state) {
  if (id_ == 0) GCore::natApp()->freezeCursor(state);
  //else TODO;
}

void GCursor::displayAsNative(bool val) {
  if (id_ != 0) nativecursor_ = val;
}

void GCursor::drawCursors(GRenderGraphics& g) {
  for (auto& c : GCore::channels()) {
    if (c) {
      if (c->shown_ && !c->nativecursor_) {
        auto* win = g.window();
        g.setWinClip(win->frame_);
        g.setColor(c->color_);
        g.unsetBgColor();
        float x = c->pos_.x - win->x();
        float y = c->pos_.y - win->y();
        g.drawLine(x-5, y, x+5, y);
        g.drawLine(x, y-5, x, y+5);
      }
    }
  }
}

bool GCursor::isDragging() const {
  return dragging_;
}

Gadget* GCursor::source() {
  return fromg_;
}

GWindow* GCursor::sourceWindow() {
  return fromwin_;
}

void GCursor::reset() {
  dragging_ = false;
  fromg_ = nullptr;
  target_ = nullptr;
  fromwin_ = nullptr;
  targetwin_ = nullptr;
  onend_ = nullptr;
}

void GCursor::startDrag(Gadget& from,
                        std::function<void(GCursor&)> const& onEnd,
                        Shape shape) {
  if (dragging_) return;
  dragging_ = true;
  fromg_ = &from;
  fromwin_ = from.getWindow();
  target_ = nullptr;
  targetwin_ = nullptr;
  onend_ = onEnd;
  GCore::natApp()->grabCursor(true);
  setShape(shape); 
}

void GCursor::endDrag(GPoint const& wpos) {
  if (!dragging_) return;
  dragging_ = false;
  GCore::natApp()->grabCursor(false);
  setShape(GCursor::Arrow);
  
  // wpos coordinates relative to fromwin_
  if (fromwin_ && wpos.x >= 0 && wpos.y >= 0   // same window
      && wpos.x < fromwin_->w()
      && wpos.y < fromwin_->h()) {
    targetwin_ = fromwin_;
    targetpos_ = wpos;
  }
  else {                                          // other window
    auto scrpos = GCursor::pos();
    targetwin_ = GWindow::findWindow(scrpos);
    //cerr << "pos " << scrpos  << " / win " << impl_.towin_ << endl;
    if (targetwin_) {
      targetpos_.x = scrpos.x - targetwin_->x();
      targetpos_.y = scrpos.y - targetwin_->y();
    }
  }
  
  if (onend_) (onend_)(*this);
  reset();
}

void GCursor::moveDrag(GPoint const& wpos) {
}

}
