//
//  Base of GObjects (useful for srcipts among other things)
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gobjectbase.hpp>
#include <guit.hpp>
#include <gchart.hpp>
#include <gcombobox.hpp>
#include <gshape.hpp>
#include <gmodes.hpp>
using namespace std;
namespace guit {

GObjectBase::GObjectBase() {}

GObjectBase::~GObjectBase() {}

GObject* GObjectBase::getObject(GString const& ident) {
  auto it = map_.find(ident);
  if (it == map_.end()) return nullptr;
  else return it->second;
}

Gadget* GObjectBase::getGadget(GString const& name) {
  auto* obj = getObject(name);
  return obj ? obj->toGadget() : nullptr;
}

GBox* GObjectBase::getBox(GString const& name) {
  auto* obj = getObject(name);
  return obj ? obj->toBox() : nullptr;
}

GProp* GObjectBase::getProp(GString const& name) {
  auto* obj = getObject(name);
  return obj ? obj->toProp() : nullptr;
}

GObjectBase::Status GObjectBase::add(GString const& name, GObject& obj) {
  bool ok = true;
  for (auto& c : name) {
    if (!::isalnum(c) && c != '_' && c != '.') {ok = false; break;}
  }
  if (!ok || name.empty() || !::isalpha(name[0])) {
    gerror("GObjectBase::addElem", name+" is not a valid identifier");
    return Status::InvalidName;
  }
  auto it = map_.find(name);
  if (it != map_.end()) {
    gerror("GObjectBase::addElem", name+" already exists (can't be redefined)");
    return Status::NameExists;
  }
  map_[name] = obj;
  return Status::OK;
}

GObjectBase::Status GObjectBase::add(GString const& name, GFunction const& f) {
  return add(name, *new GFunctionWrapper(f));
}

void GObjectBase::printObjects(std::ostream& out) {
  for (auto & it : map_) out << it.first << ", " ;
}

GString GFunctionWrapper::typeName() const {
  return "/F";
}

void GObjectBase::loadGadgetClasses() {
  static bool loaded(false);
  if (!loaded) {
    loaded = true;

    GLabel::Type();
    GFiller::Type();
    GSeparator::Type();

    GButton::Type();
    GItem::Type();
    GLinkItem::Type();
    GCheckItem::Type();
    GRadioItem::Type();
    GChoiceItem::Type();

    GSlider::Type();
    GScrollBar::Type();

    GTextField::Type();
    GTextArea::Type();

    GBox::Type();
    GVBox::Type();
    GHBox::Type();
    //GPane::Type();
    GMenuBar::Type();
    GToolBar::Type();
    GStatusBar::Type();

    GChoiceBox::Type();
    GListBox::Type();
    GRadioBox::Type();
    GStackBox::Type();
    GTabBox::Type();
    GOptionBox::Type();
    GComboBox::Type();

    GTable::Type();
    GTRow::Type();
    GTHead::Type();

    GMenu::Type();
    GWindow::Type();
    GDialog::Type();

    GInfoDialog::Type();
    GQuitDialog::Type();

    GFileDialog::Type();
    GOpenDialog::Type();
    GSaveDialog::Type();

    GChartBox::Type();
    GChartItem::Type();

    // Shapes.....
  }
}

}
