//
// Textual format
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gbox.hpp>
#include <gformat.hpp>
#include <gcontrol.hpp>
#include <grender.hpp>
#include <gstring.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_PROP_TYPE(Format,GPropType::Format)

GFormatValue::GFormatValue() :
 precision{-1}, nbchars{-1}, fillchar{'0'}
//decoration{},
//lineheight{0}   // should be inherited
{}

bool GFormatValue::operator==(const GFormatValue& other) const {
  return
  nbchars == other.nbchars
  && precision == other.precision
  && fillchar == other.fillchar;
  /*
  && decoration.underline == other.decoration.underline
  && decoration.overline == other.decoration.overline
  && decoration.linethrough == other.decoration.linethrough
  && lineheight == other.lineheight;
   */
}

bool GFormatValue::operator!=(const GFormatValue& other) const {
  return
  nbchars != other.nbchars
  || precision != other.precision
  || fillchar != other.fillchar;
  /*
  || decoration.underline != other.decoration.underline
  || decoration.overline != other.decoration.overline
  || decoration.linethrough != other.decoration.linethrough
  || lineheight != other.lineheight;
   */
}

bool gconvert(GFormatValue const& from, GFormatValue& to) {
  to = from;
  return true;
}

bool gconvert(GString const& from, GFormatValue& to) {
  std::vector<GString> v;
  auto n = strsplit(from, v, "");

  GString curloc = setlocale(LC_ALL,NULL);
  setlocale(LC_NUMERIC, "C");

  if (n > 0) {
    to.precision = (int8_t) std::atoi(v[0].data());
  }
  if (n > 1) {
    to.nbchars = (int8_t) std::atoi(v[1].data());
  }
  if (n > 2 && !v[2].empty()) {
    to.fillchar = v[2][0];
  }
  return true;
}

bool gconvert(GFormatValue const& from, GString& to) {
  to =
  to_string(from.precision)
  + " " + to_string(from.nbchars)
  + " " + from.fillchar;
  return true;
}

// - - - - - - - - - -

GFormat::GFormat(GString const& value) {
  set(value);
}

GFormat::GFormat(GFormatValue const& value) : Super(value) {}

GFormat::GFormat(GFormat const& from) : Super(from.value_) {}

GFormat* GFormat::clone(bool copy_value) const {
  return copy_value ? new GFormat(*this) : new GFormat;
}

void GFormat::updateGadget(Gadget& g) {
  g.updateLayout();
}

void GFormat::apply(Gadget*, GRender& r, Specif specif) {
  if (specif < r.specif[GPropType::Format]) return;
  r.specif[GPropType::Format] = specif;

  r.format_ = &value_;
}

}
