//
// Icon of a gadget
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_Icon_hpp
#define Guit_Icon_hpp
#include <gcolor.hpp>
namespace guit {

class GIconValue {
public:
  enum LoadStatus {CantLoad=-1, Unloaded=0, Loaded=1};
  typedef void (DrawFun)(GRenderGraphics&, GRender&, GRect const& iconframe);

  ~GIconValue();
  bool operator==(const GIconValue&) const;
  bool operator!=(const GIconValue&) const;

  mutable LoadStatus status{Unloaded};
  mutable GTexture texture{};
  DrawFun* drawfun{};
  GString filename;
  const char** xpm{};
  gptr<GColor> fg, bg;
  int w{}, h{}, area{};
};

bool gconvert(GIconValue const& from, GIconValue& to);
bool gconvert(GString const& from, GIconValue& to);
bool gconvert(GIconValue const& from, GString& to);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

/// Icon prop.
class GIcon : public GVarProp_<GIconValue> {
public:
  using Super = GVarProp_<GIconValue>;
  using DrawFun = GIconValue::DrawFun;
  static DrawFun Image, XPM,
  CheckSymbol, RadioSymbol, ComboSymbol, LeftSymbol, RightSymbol, UpSymbol,
  DownSymbol, UpDownSymbol;

  /// predefined icons.
  static GIcon check, radio, left, right, up, down, updown, combo;

  GIcon();
  GIcon(GString const& filename);
  GIcon(const char** xpm);
  GIcon(DrawFun, GColor& fg, GColor& bg);
  GIcon(GIconValue const& value);
  GIcon* clone(bool copy_value) const override;

  /// Future value in an Active Expression.
  /// An Active Expression (GExpr) can refer ~prop.
  GPropRef operator~() {return GPropRef(*this);}

  using GType = GPropType_<GIcon,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}

  void apply(Gadget*, GRender&, Specif) override;
  void updateGadget(Gadget&) override;
  static void drawHArrow(GRenderGraphics& g, GRect const& frame,
                         float lx, float rx, float ty, float by);
  static void drawVArrow(GRenderGraphics& g, GRect const& frame,
                         float lx, float rx, float ty, float by);
};

/// Returns a GIcon created in the heap.
/// @{
inline GIcon& Icon() {return *new GIcon();}
inline GIcon& Icon(GString const& filename) {return *new GIcon(filename);}
/// @}

}
#endif
